import logging
from typing import Any, Optional

import requests

from pagerduty_api import log_it
from pagerduty_api.abstract_client import AbstractClient


class PagerDutyEscalationPoliciesClient(AbstractClient):
    def __init__(
        self,
        http_session: requests.Session = requests.Session(),
        endpoint: str = "https://api.pagerduty.com",
        auth_token: Optional[str] = None,
        logger: Optional[logging.Logger] = None,
    ):
        super().__init__(http_session, endpoint, auth_token, logger)
        self._default_url = f"{self._endpoint}/escalation_policies"
        self._default_entity_name = "escalation_policies"

    @log_it
    def list_escalation_policies(
        self, limit: int = 100, params: Optional[dict[str, Any]] = None, paginate: bool = True
    ):
        """
        >>> import os
        >>> client = PagerDutyEscalationPoliciesClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> policies = client.list_escalation_policies()
        Executing HTTP get on https://api.pagerduty.com/escalation_policies
        >>> len(policies) > 50
        True
        """
        if not params:
            params = {
                "query": "",
                "sort_by": "name",
                "limit": limit,
            }
        return self.do_request(
            url=self._default_url,
            entity_name=self._default_entity_name,
            params=params,
            paginate=paginate,
        )

    @log_it
    def get_escalation_policies_by_name(self, policy_name: str):
        """
        >>> import os
        >>> client = PagerDutyEscalationPoliciesClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> policies = client.get_escalation_policies_by_name('AISO-Tech')
        Executing HTTP get on https://api.pagerduty.com/escalation_policies
        >>> len(policies) == 1
        True
        """
        return self.do_request(
            url=self._default_url,
            entity_name=self._default_entity_name,
            params={
                "user_ids[]": [],
                "time_zone": "UTC",
                "sort_by": "name",
                "query": str(policy_name),
                "include[]": [],
            },
        )



import logging
from typing import Optional

import requests

from pagerduty_api import log_it
from pagerduty_api.abstract_client import AbstractClient


class PagerDutyUsersClient(AbstractClient):

    def __init__(
        self,
        http_session: requests.Session = requests.Session(),
        endpoint: str = "https://api.pagerduty.com",
        auth_token: Optional[str] = None,
        logger: Optional[logging.Logger] = None,
    ):
        super().__init__(http_session, endpoint, auth_token, logger)
        self._default_url = f"{self._endpoint}/users"
        self._default_entity_name = "users"

    @log_it
    def list_users(self, team_ids: list[str] = [], limit: int = 100, paginate: bool = True):
        """
        >>> import os
        >>> client = PagerDutyUsersClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> users = client.list_users(paginate=False)
        Executing HTTP get on https://api.pagerduty.com/users
        >>> len(users) > 0
        True
        """
        return self.do_request(
            url=self._default_url,
            entity_name=self._default_entity_name,
            params={
                "team_ids[]": team_ids,
                "time_zone": "UTC",
                "sort_by": "name",
                "query": "",
                "limit": limit,
                "include[]": [],
            },
            paginate=paginate,
        )

    @log_it
    def list_users_by_team(self, team_ids: list[str] = [], limit: int = 100, paginate: bool = True):
        """
        >>> import os
        >>> client = PagerDutyUsersClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> users_by_team = client.list_users_by_team(team_ids=['PUE38G0'])
        Executing HTTP get on https://api.pagerduty.com/users
        >>> len(users_by_team) == 1
        True
        """
        users_dict = {}  # team ID -> list of users
        for user in self.list_users(team_ids, limit, paginate):
            for team in user["teams"]:
                if team["id"] not in team_ids:
                    continue
                if team["id"] not in users_dict:
                    users_dict[team["id"]] = []
                users_dict[team["id"]].append(user)
        return users_dict

    @log_it
    def get_user_by_id(self, user_id: str):
        """
        >>> import os
        >>> client = PagerDutyUsersClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> users = client.get_user_by_id('PN5ECJH')
        Executing HTTP get on https://api.pagerduty.com/users/PN5ECJH
        >>> len(users) > 0
        True
        """
        return self.do_request(url=f"{self._default_url}/{user_id}", entity_name="user")

    @log_it
    def get_user_by_email(self, email: str):
        """
        >>> import os
        >>> client = PagerDutyUsersClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> users = client.get_user_by_email('john.torres@pearson.com')
        Executing HTTP get on https://api.pagerduty.com/users
        >>> len(users) > 0
        True
        """
        return self.do_request(
            url=self._default_url,
            entity_name=self._default_entity_name,
            params={
                "user_ids[]": [],
                "time_zone": "UTC",
                "sort_by": "name",
                "query": str(email),
                "include[]": [],
            },
        )

    @log_it
    def is_valid_user(self, email: str):
        """
        >>> import os
        >>> client = PagerDutyUsersClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> b = client.is_valid_user('john.torres@pearson.com')
        Executing HTTP get on https://api.pagerduty.com/users
        >>> b
        True
        """
        users = self.get_user_by_email(email)
        return len(users) > 0



import logging
from typing import Any, Optional

import requests

from pagerduty_api import log_it
from pagerduty_api.abstract_client import AbstractClient
from pagerduty_api.utils import is_service_name_compliant


class PagerDutyServicesClient(AbstractClient):

    def __init__(
        self,
        http_session: requests.Session = requests.Session(),
        endpoint: str = "https://api.pagerduty.com",
        auth_token: Optional[str] = None,
        logger: Optional[logging.Logger] = None,
    ):
        super().__init__(http_session, endpoint, auth_token, logger)
        self._default_url = f"{self._endpoint}/services"
        self._default_entity_name = "services"

    @log_it
    def list_services(self, limit: int = 100, params: dict[str, Any] = {}, paginate: bool = True):
        """
        >>> import os
        >>> client = PagerDutyServicesClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> services = client.list_services()
        Executing HTTP get on https://api.pagerduty.com/services
        >>> len(services) > 250
        True
        """
        if not params:
            params = {
                "query": "",
                "sort_by": "name",
                "limit": limit,
            }
        params["limit"] = params.get("limit", limit)
        return self.do_request(
            url=self._default_url,
            entity_name=self._default_entity_name,
            params=params,
            paginate=paginate,
        )

    # def list_compliant_services(
    #     self,
    #     limit: int = 100,
    #     params: dict[str, Any] = {},
    #     paginate: bool = True
    # ):
    #     '''
    #     >>> import os
    #     >>> client = PagerDutyServicesClient(auth_token=os.environ.get('AUTH_TOKEN'))
    #     >>> services = client.list_compliant_services()
    #     Executing HTTP get on https://api.pagerduty.com/services
    #     >>> len(services) > 200
    #     True
    #     '''
    #     services = self.list_services(limit, params, paginate)
    #     return [
    #         s for s in services
    #         if is_service_name_compliant(s['name'])
    #     ]

    def list_compliant_services(self, services: list = [dict[str, Any]]):
        """
        >>> import os
        >>> client = PagerDutyServicesClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> services = client.list_compliant_services()
        Executing HTTP get on https://api.pagerduty.com/services
        >>> len(services) > 200
        True
        """
        self._logger.debug("Listing compliant services")
        return [s for s in services if is_service_name_compliant(s["name"])]

        # return service ids
        # return service names



import logging
from typing import Any, Optional

import requests

from pagerduty_api import log_it
from pagerduty_api.abstract_client import AbstractClient


class PagerDutyIncidentsClient(AbstractClient):

    def __init__(
        self,
        http_session: requests.Session = requests.Session(),
        endpoint: str = "https://api.pagerduty.com",
        auth_token: Optional[str] = None,
        logger: Optional[logging.Logger] = None,
    ):
        super().__init__(http_session, endpoint, auth_token, logger)
        self._default_url = f"{self._endpoint}/incidents"
        self._default_entity_name = "incidents"

    @log_it
    def list_incidents(self, limit: int = 100, params: dict[str, Any] = {}, paginate: bool = True):
        """
        >>> import os
        >>> client = PagerDutyIncidentsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> incidents = client.list_incidents()
        Executing HTTP get on https://api.pagerduty.com/incidents
        >>> len(incidents) >= 1
        True
        """
        if not params:
            params = {
                "query": "",
                "sort_by": "name",
                "limit": limit,
            }
        params["limit"] = params.get("limit", limit)
        return self.do_request(
            url=self._default_url,
            entity_name=self._default_entity_name,
            params=params,
            paginate=paginate,
        )



import json
import logging
from typing import Any, Optional

import requests

from pagerduty_api import log_it
from pagerduty_api.abstract_client import AbstractClient


class PagerDutyTagsClient(AbstractClient):
    def __init__(
        self,
        http_session: requests.Session = requests.Session(),
        endpoint: str = "https://api.pagerduty.com",
        auth_token: Optional[str] = None,
        logger: Optional[logging.Logger] = None,
    ):
        super().__init__(http_session, endpoint, auth_token, logger)
        self._default_url = f"{self._endpoint}/tags"
        self._default_entity_name = "tags"

    @log_it
    def list_tags(self, limit: int = 100, params: dict[str, Any] = {}, paginate: bool = True):
        """
        >>> import os
        >>> client = PagerDutyTagsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> tags = client.list_tags()
        Executing HTTP get on https://api.pagerduty.com/tags
        >>> len(tags) > 0
        True
        """
        if not params:
            params = {
                "query": "",
                "sort_by": "name",
                "limit": limit,
            }
        params["limit"] = params.get("limit", limit)
        return self.do_request(
            url=self._default_url,
            entity_name=self._default_entity_name,
            params=params,
            paginate=paginate,
        )

    @log_it
    def get_tag(self, tag_id: str, params: dict[str, Any] = {}):
        """
        >>> import os
        >>> client = PagerDutyTagsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> tags = client.get_tag('PKYLUO6')
        Executing HTTP get on https://api.pagerduty.com/tags/PKYLUO6
        >>> len(tags) > 0
        True
        """
        assert tag_id, f"Cannot get tag with tag id: {tag_id}"
        resp = self.do_request(
            url=f"{self._default_url}/{tag_id}", entity_name="tag", params=params
        )
        tag = None
        if len(resp) == 1:
            tag = resp[0]
        return tag

    @log_it
    def create_tag(self, label: str = "", summary: str = ""):
        """No doctest so we don't affect state in production PagerDuty"""
        url = f"{self._endpoint}/tags"
        data = {"tag": {"type": "tag", "label": str(label), "summary": str(summary)}}
        resp = self.do_request(
            url=self._default_url, entity_name="tag", method="post", data=json.dumps(data)
        )
        if len(resp) == 0:
            raise Exception("Failed to create tag")
        return resp[0]

    @log_it
    def delete_tag(self, tag_id: str = ""):
        """No doctest so we don't affect state in production PagerDuty"""
        return self.do_request(
            url=f"{self._default_url}/{tag_id}", entity_name="tag", method="delete"
        )



from typing import Optional


def is_service_name_compliant(service_name: str) -> bool:
    # TODO: use a regex
    return service_name and "_SVC" in service_name.upper()


def get_t_appid_from_service_name(service_name: str) -> str:
    if is_service_name_compliant(service_name):
        t_appid = service_name.split("_SVC")[-1]
        return f"SVC{t_appid}"
    return None


def dedup_entities(entities: list[dict]) -> list[dict]:
    entities_dict = {}
    for entity in entities:
        entities_dict[entity["id"]] = entity
    return list(entities_dict.values())


def build_resolved_service_dict(service, team_members: Optional[list[dict]] = None):
    if not is_service_name_compliant(service["name"]):
        return None

    t_appid = get_t_appid_from_service_name(service["name"])
    name = service["name"].split("_SVC")[0]

    return {
        "t_appid": t_appid,
        "id": service["id"],
        "name": name,
        "teams": service["teams"],
        "sre": team_members,
        "service": {
            "id": service["id"],
            "type": "service_reference",
            "summary": service["name"],
            "self": service["self"],
            "html_url": service["html_url"],
        },
    }



import logging
from datetime import datetime, timezone, timedelta
from typing import Any, Optional

import requests

from pagerduty_api import log_it
from pagerduty_api.abstract_client import AbstractClient


class PagerDutyChangeEventsClient(AbstractClient):

    def __init__(
        self,
        http_session: requests.Session = requests.Session(),
        endpoint: str = "https://api.pagerduty.com",
        auth_token: Optional[str] = None,
        logger: Optional[logging.Logger] = None,
    ):
        super().__init__(http_session, endpoint, auth_token, logger)
        self._default_url = f"{self._endpoint}/change_events"
        self._default_entity_name = "change_events"

    @log_it
    def list_change_events(
        self,
        limit: int = 100,
        since: str = (
            datetime.now(timezone.utc).replace(hour=0, minute=0, second=0, microsecond=0)
            - timedelta(days=1)
        ).isoformat(),
        params: dict[str, Any] = {},
        paginate: bool = True,
    ):
        """
        Links to docs: https://developer.pagerduty.com/api-reference/c36fffe46ba3b-list-change-events
        """
        if not params:
            params = {"query": "", "sort_by": "name", "limit": limit, "since": since}
        params["limit"] = params.get("limit", limit)
        params["since"] = params.get("since", since)
        return self.do_request(
            url=self._default_url,
            entity_name=self._default_entity_name,
            params=params,
            paginate=paginate,
        )



import logging
from typing import Any, Optional

import requests

from pagerduty_api import log_it
from pagerduty_api.abstract_client import AbstractClient


class PagerDutyAnalyticsClient(AbstractClient):

    def __init__(
        self,
        http_session: requests.Session = requests.Session(),
        endpoint: str = "https://api.pagerduty.com",
        auth_token: Optional[str] = None,
        logger: Optional[logging.Logger] = None,
    ):
        super().__init__(http_session, endpoint, auth_token, logger)
        self._default_url = f"{self._endpoint}/analytics"
        self._default_entity_name = "data"

    @log_it
    def list_aggregated_services(
        self, filters: dict[str, Any] = {}, aggregate_unit: Optional[str] = None
    ):
        """
        >>> import os
        >>> client = PagerDutyAnalyticsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> analytics = client.list_aggregated_services(data=data)
        Executing HTTP get on https://api.pagerduty.com/metrics/incidents/services
        >>> len(analytics) >= 1
        True
        """
        return self.do_analytics_request(
            url=f"{self._default_url}/metrics/incidents/services",
            entity_name=self._default_entity_name,
            filters=filters,
            method="post",
            aggregate_unit=aggregate_unit,
        )

    @log_it
    def list_aggregated_teams(
        self, filters: dict[str, Any] = {}, aggregate_unit: Optional[str] = None
    ):
        """
        >>> import os
        >>> client = PagerDutyAnalyticsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> analytics = client.list_aggregated_teams(data=data)
        Executing HTTP get on https://api.pagerduty.com/metrics/incidents/teams
        >>> len(analytics) >= 1
        True
        """
        return self.do_analytics_request(
            url=f"{self._default_url}/metrics/incidents/teams",
            entity_name=self._default_entity_name,
            filters=filters,
            method="post",
            aggregate_unit=aggregate_unit,
        )

    @log_it
    def list_aggregated_incidents(self, filters: dict[str, Any] = {}):
        """
        >>> import os
        >>> client = PagerDutyAnalyticsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> incidents = client.list_aggregated_incidents(data=data)
        Executing HTTP get on https://api.pagerduty.com/incidents
        >>> len(incidents) >= 1
        True
        """

        return self.do_analytics_request(
            url=f"{self._default_url}/metrics/incidents/all",
            entity_name=self._default_entity_name,
            filters=filters,
            method="post",
        )

    @log_it
    def list_raw_incidents(
        self, filters: dict[str, Any] = None, limit: Optional[int] = None, paginate: bool = False
    ):
        """
        >>> import os
        >>> client = PagerDutyAnalyticsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> incidents = client.list_raw_incidents(data=data)
        Executing HTTP get on https://api.pagerduty.com/analytics/raw/incidents
        >>> len(incidents) >= 1
        True
        """

        return self.do_analytics_request(
            url=f"{self._default_url}/raw/incidents",
            entity_name=self._default_entity_name,
            filters=filters or {},
            method="post",
            limit=limit,
            paginate=paginate,
        )


# ["P6THBO4", "PAOR13M", "PCTB88X", "PYPJAA9", "PG2XNDN", "PQI2LK3", "PNHSHMO", "PEG1GVL", "PRX56PU", "PG65ZZE"]
# feed service ids in batches of 10
# get incident count from here



import json
import logging
from typing import Any, Optional

import requests

from pagerduty_api import log_it
from pagerduty_api.abstract_client import AbstractClient


class PagerDutyTeamsClient(AbstractClient):

    def __init__(
        self,
        http_session: requests.Session = requests.Session(),
        endpoint: str = "https://api.pagerduty.com",
        auth_token: Optional[str] = None,
        logger: Optional[logging.Logger] = None,
    ):
        super().__init__(http_session, endpoint, auth_token, logger)
        self._default_url = f"{self._endpoint}/teams"
        self._default_entity_name = "teams"

    @log_it
    def list_teams(self, limit: int = 100, paginate: bool = True):
        """
        >>> import os
        >>> client = PagerDutyTeamsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> teams = client.list_teams()
        Executing HTTP get on https://api.pagerduty.com/teams
        >>> len(teams) > 0
        True
        """
        return self.do_request(
            url=self._default_url,
            entity_name=self._default_entity_name,
            params={"limit": limit},
            paginate=paginate,
        )

    @log_it
    def get_team(
        self, team_id: str, with_members: bool = False, params: Optional[dict[str, Any]] = None
    ):
        """
        >>> import os
        >>> client = PagerDutyTeamsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> teams = client.get_team('PUE38G0')
        Executing HTTP get on https://api.pagerduty.com/teams/PUE38G0
        >>> len(teams) > 0
        True
        >>> teams = client.get_team('PUE38G0', with_members=True)
        Executing HTTP get on https://api.pagerduty.com/teams/PUE38G0
        Executing HTTP get on https://api.pagerduty.com/teams/PUE38G0/members
        >>> len(teams['members']) > 0
        True
        """
        resp = self.do_request(
            url=f"{self._default_url}/{team_id}", entity_name="team", params=params
        )
        if len(resp) == 0:
            return None

        team = resp[0]
        if with_members:
            team["members"] = self.get_team_members(team_id)
        return team

    @log_it
    def get_team_members(self, team_id: str):
        """
        >>> import os
        >>> client = PagerDutyTeamsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> members = client.get_team_members('PUE38G0')
        Executing HTTP get on https://api.pagerduty.com/teams/PUE38G0/members
        >>> len(members) > 0
        True
        """
        return self.do_request(url=f"{self._default_url}/{team_id}/members", entity_name="members")

    @log_it
    def is_member_of_team(self, team_id: str = "", user_id: str = "") -> bool:
        """
        >>> import os
        >>> client = PagerDutyTeamsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> b = client.is_member_of_team('PUE38G0', 'PN5ECJH')
        Executing HTTP get on https://api.pagerduty.com/teams/PUE38G0/members
        >>> b
        True
        """
        for user in self.get_team_members(team_id):
            if user["user"]["id"] == user_id:
                return True
        return False

    @log_it
    def create_team(self, name: str = "", description: str = "", summary: str = ""):
        """No doctest so we don't affect state in production PagerDuty"""
        data = {
            "type": "team",
            "name": str(name),
            "description": str(description),
            "summary": str(summary),
        }
        resp = self.do_request(
            url=self._default_url, entity_name="team", method="post", data=json.dumps(data)
        )
        if len(resp) == 0:
            raise Exception("Failed to create team")
        return resp[0]

    @log_it
    def delete_team(self, team_id: str):
        """No doctest so we don't affect state in production PagerDuty"""
        return self.do_request(
            url=f"{self._default_url}/{team_id}", entity_name="team", method="delete"
        )



import json
import logging
from typing import Any, Optional

import requests

from pagerduty_api import get_logger, log_it


class AbstractClient:
    def __init__(
        self,
        http_session: requests.Session = requests.Session(),
        endpoint: str = "https://api.pagerduty.com",
        auth_token: Optional[str] = None,
        logger: Optional[logging.Logger] = None,
    ):
        self._endpoint = endpoint
        self._http_session = http_session
        self._auth_token = auth_token
        self._valid_methods = ["get", "patch", "post", "put", "head", "delete"]
        self._logger = logger or get_logger()

    @log_it
    def do_request(
        self,
        url: str,
        entity_name: str,
        headers: dict[str, str] = {},
        params: dict[str, Any] = {},
        paginate: bool = True,
        method: str = "get",
        data: Optional[Any] = None,
    ) -> list[Any]:
        assert url, f"Cannot do HTTP request against url: {url}"
        assert entity_name, f"Cannot do HTTP request with entity: {entity_name}"
        assert method in self._valid_methods, f'"{method}" is not a valid request method'
        headers = self._set_default_headers(headers)
        params = self._set_default_params(params)
        pages = {"more": True, "limit": params["limit"], "offset": params["offset"]}

        # FIXME: use a logger
        # print(f'Executing HTTP {method} on {url}')

        records = []
        while pages.get("more"):  # we always make at least one call...
            response = getattr(self._http_session, method)(
                url, headers=headers, params=params, data=data
            )

            # self._logger.info(response.request.headers)
            # self._logger.info(f'{response.request.method} {response.request.url}')
            if response.status_code in [204, 400, 404, 500]:  # return no content
                # FIXME: use a logger
                print(
                    f"HTTP response returned with status {response.status_code}. Returning empty records."
                )
                return records

            pages = response.json()
            if "offset" in pages:
                records.extend(pages[entity_name])
                params["offset"] = pages["offset"] + pages["limit"]
            elif "ok" not in pages:
                records.append(pages[entity_name])

            if not paginate or method != "get":  # disable pagination
                break

        return records

    @log_it
    def do_analytics_request(
        self,
        url: str,
        entity_name: str,
        filters: dict[str, Any] = {},
        method: str = "post",
        limit: Optional[int] = None,
        paginate: bool = False,
        aggregate_unit: Optional[str] = None,
    ) -> list[Any]:
        assert url, f"Cannot do HTTP request against url: {url}"
        assert entity_name, f"Cannot do HTTP request with entity: {entity_name}"
        assert method in self._valid_methods, f'"{method}" is not a valid request method'
        method = method.upper()

        headers = {
            "x-early-access": "analytics-v2",
            "accept": "application/vnd.pagerduty+json;version=2",
            "content-type": "application/json",
            "authorization": f"Token token={self._auth_token}",
        }
        payload = self._set_default_analytics_data(
            filters=filters,
            aggregate_unit=aggregate_unit if aggregate_unit else None if "raw" in url else "day",
            limit=limit,
        )

        data, more = [], True
        while more:
            self._logger.info(
                f"Executing {method} on {url} with data {payload} and headers {headers}"
            )
            response = requests.request(method, url, data=json.dumps(payload), headers=headers)
            if response.status_code in [204, 400, 404, 500]:  # return no content
                self._logger.error(
                    f"HTTP response returned with status {response.status_code}. Returning empty records."
                )
                return []
            response.raise_for_status()
            resp_json = response.json()
            data.extend(resp_json.get(entity_name) or [])
            more = paginate and resp_json.get("more") and resp_json.get("last")
            if more:
                payload["starting_after"] = resp_json["last"]
        return data

    @log_it
    def _set_default_headers(self, headers: dict[str, str]) -> dict[str, str]:
        """Set the default Accept, Authorization, and Content-type headers if not already present"""
        cp = {} if headers is None else headers.copy()
        cp["Accept"] = cp.get("Accept", "application/vnd.pagerduty+json;version=2")
        cp["Authorization"] = cp.get("Authorization", f"Token token={self._auth_token}")
        cp["Content-type"] = cp.get("Content-type", "application/json")
        return cp

    @log_it
    def _set_default_params(self, params: dict[str, Any]) -> dict[str, Any]:
        """Set the default limit and offset params if not already present"""
        cp = {} if params is None else params.copy()
        cp["limit"] = cp.get("limit", 100)
        cp["offset"] = cp.get("offset", 0)
        return cp

    @log_it
    def _set_default_analytics_data(
        self,
        filters: dict[str, Any],
        aggregate_unit: str = "day",
        urgency: str = "high",
        limit: Optional[int] = None,
    ) -> dict[str, Any]:
        """Set the default limit and offset params if not already present"""
        data = {"filters": filters}
        if urgency:
            data["filters"] = data["filters"] or {}
            data["filters"]["urgency"] = urgency
        if aggregate_unit:
            data["aggregate_unit"] = aggregate_unit
        if limit:
            data["limit"] = limit
        return data

    @log_it
    def get_entity_tags(self, url: str, entity_id: str, params: Optional[dict[str, Any]] = None):
        assert entity_id, f"Cannot get entity tags with entity id: {entity_id}"
        url = url if url else f"{self._endpoint}/{self._default_entity_name}"
        resp = self.do_request(url=f"{url}/{entity_id}/tags", entity_name="tags", params=params)

        tag = None
        if len(resp) == 1:
            tag = resp[0]
        return tag

    @log_it
    def modify_entity_tags(self, url: str, entity_id: str, assignments: dict[Any, Any] = None):
        assert url, f"Cannot modify entity tags with url: {url}"
        assert entity_id, f"Cannot modify entity tags with entity id: {entity_id}"
        return self.do_request(
            url=f"{url}/{entity_id}/change_tags",
            entity_name="tag",
            method="post",
            data=json.dumps(assignments),
        )
